==> sagua_0.2.0-1.tar.gz: sagua-0.2.0/src/mixer.cc <==
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file mixer.cc
/// @brief Class for OSS mixer source file

#include <fcntl.h>
#include <sys/ioctl.h>

#include "../include/mixer.h"
#include "../include/mainmanager.h"
#include "../include/scopeguard.h"

/// @brief Get current PCM volumn from specific mixer device.
///
/// Get current PCM volumn and return in double. If left is
/// diffrent from right volumn, return larger one. If fail to open device
/// or cannot control device, return currentPCMVolumn_(initial value is 0.0).
/// @return Current PCM volumn in double
const double& Mixer::GetVolumn()
{
    int fd = open(mainManager_.MixerDev().c_str(), O_RDONLY);
    ScopeGuard fileClose = MakeGuard(close, fd);

    if(fd != -1) {
	int dev = 0, cmd = 0;
	
	if(ioctl(fd, SOUND_MIXER_READ_DEVMASK, &dev) -1) {
	    
	    if(dev && SOUND_MASK_PCM)
		cmd = SOUND_MIXER_READ_PCM;
	    else
		return currentPCMVolumn_;
	
	    int volumn = 0, left = 0, right = 0;
	    if(ioctl(fd, cmd, &volumn) != -1) {
	
		left = (volumn & 0xFF00) >> 8;
		right = (volumn & 0x00FF);

		if(left >= right)
		    currentPCMVolumn_ = left;
		else
		    currentPCMVolumn_ = right;
	    }

	}
    } else
	fileClose.Dismiss();

    return currentPCMVolumn_;
}

/// @brief Set PCM volumn to specific mixer device.
///
/// Set PCM volumn to specific mixer device, same level to left and right.
/// Parameter must over than 0 and under 100 in double. Other value will be
/// rejected.
/// @param volumn Wanted volumn in double
void Mixer::SetVolumn(const double& volumn)
{
    if(volumn < 0.0 || volumn > 100.0)
	return;

    int fd = open(mainManager_.MixerDev().c_str(), O_RDONLY);
    ScopeGuard fileClose = MakeGuard(close, fd);

    if(fd != -1) {
	int dev = 0, cmd = 0;
	
	if(ioctl(fd, SOUND_MIXER_READ_DEVMASK, &dev) != -1) {
	    if(dev && SOUND_MASK_PCM)
		cmd = SOUND_MIXER_WRITE_PCM;
	    else
		return;

	    int destVolumn = static_cast<int>(volumn);
	    destVolumn = (destVolumn << 8) | destVolumn; 
	
	    ioctl(fd, cmd, &destVolumn);
	}

    } else
	fileClose.Dismiss();
}

==> sagua_0.2.3-1.tar.gz: sagua-0.2.3/src/mixer.cc <==
/* -*- Mode: C++; tab-width: 4; indent-tabs-mod: nil; c-basic-offset: 4 -*- */
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file mixer.cc
/// @brief Class for OSS mixer source file

#include <fcntl.h>
#include <sys/ioctl.h>

#include "../include/mixer.h"
#include "../include/mainmanager.h"
#include "../include/scopeguard.h"

/// @brief Get current PCM volumn from specific mixer device.
///
/// Get current PCM volumn and return in double. If left is
/// diffrent from right volumn, return larger one. If fail to open device
/// or cannot control device, return currentPCMVolumn_(initial value is 0.0).
/// @return Current PCM volumn in double
const double& Mixer::GetVolumn()
{
    int fd = open(mainManager_.MixerDev().c_str(), O_RDONLY);
    if(fd == -1)
        return currentPCMVolumn_;

    ScopeGuard fileClose = MakeGuard(close, fd);

    int dev = 0;
    if(ioctl(fd, SOUND_MIXER_READ_DEVMASK, &dev) != -1) {

        int cmd = 0;
	    
        if(dev && SOUND_MASK_PCM)
            cmd = SOUND_MIXER_READ_PCM;
        else
            return currentPCMVolumn_;
	
        int volumn = 0;
        if(ioctl(fd, cmd, &volumn) != -1) {
	
            int left = (volumn & 0xFF00) >> 8;
            int right = (volumn & 0x00FF);

            if(left >= right)
                currentPCMVolumn_ = left;
            else
                currentPCMVolumn_ = right;
        }

    }

    return currentPCMVolumn_;
}

/// @brief Set PCM volumn to specific mixer device.
///
/// Set PCM volumn to specific mixer device, same level to left and right.
/// Parameter must over than 0 and under 100 in double. Other value will be
/// rejected.
/// @param volumn Wanted volumn in double
void Mixer::SetVolumn(const double& volumn)
{
    if(volumn < 0.0 || volumn > 100.0)
	return;

    int fd = open(mainManager_.MixerDev().c_str(), O_RDONLY);
    if(fd == -1)
        return;

    ScopeGuard fileClose = MakeGuard(close, fd);

    int dev = 0;
	
    if(ioctl(fd, SOUND_MIXER_READ_DEVMASK, &dev) != -1) {

        int cmd = 0;

        if(dev && SOUND_MASK_PCM)
            cmd = SOUND_MIXER_WRITE_PCM;
        else
            return;

        int destVolumn = static_cast<int>(volumn);
        destVolumn = (destVolumn << 8) | destVolumn; 
	
        ioctl(fd, cmd, &destVolumn);
    }
}
